import io.thp.pyotherside 1.5
import QtQuick 2.5
import QtQuick.Controls 2.1
import QtQuick.Layouts 1.1
import QtLocation 5.9
import QtPositioning 5.9
import Ubuntu.Components 1.3

// stuff to do
// - check settings for login details
// - check if logged in, popup for auth?
// - download nearby caches

MainView {
    id: root

    objectName: 'mapView'
    applicationName: 'gpstest.evilbunny'

    visible: true
    width: units.gu(50)
    height: units.gu(80)

    Page {
        id: page
        anchors.fill: parent

        header: PageHeader {
            id: header
            title: "Waiting for a GPS fix..."
        }

        ListModel {
            id: locationModel
        }

        Plugin {
            id: osmMapPlugin
            name: "osm"
            PluginParameter { name: "osm.mapping.custom.host"; value: "https://map.fosm.org/" }
            PluginParameter { name: "osm.mapping.copyright"; value: "&copy; by <a href='https://fosm.org'>FOSM.org and contributors</a>" }
        }

        Component.onCompleted: {
            // pytest.call("util.downloadAndSave", ["https://www.geocaching.com/account/signin", "geocaching.com.txt"], function(result) {
            // pytest.call("util.downloadAndSave", ["http://delungra.com", "delungra.com.txt"], function(result) {
            //     console.log(result)
            // })
        }


        Map {
            id: map

            property int lastX: -1
            property int lastY: -1
            property int pressX: -1
            property int pressY: -1
            property int jitterThreshold: 30

            plugin: osmMapPlugin
            activeMapType: supportedMapTypes[supportedMapTypes.length - 1]

            anchors.fill: parent

            center {
                // Sydney
                latitude: -33.8665593
                longitude: 151.2086631
            }
            zoomLevel: 18

            MapItemView {
                model: locationModel
                delegate: MapQuickItem {
                    id: marker
                    coordinate: QtPositioning.coordinate(lat, lon)

                    anchorPoint.x: image.width * 0.5
                    anchorPoint.y: image.height

                    sourceItem: Column {
                        Image { id: image; source: "../assets/marker.png"; width: units.gu(5); height: units.gu(5) }
                        Text { text: title; font.bold: true }
                    }

                    MouseArea {
                        id: idMyMouseArea;
                        acceptedButtons: Qt.LeftButton
                        parent: marker
                        anchors.fill: parent

                        property variant lastCoordinate

                        onPressed: {                            
                            console.log("marker was clicked, lat: " + coordinate.latitude + ", lon: " + coordinate.longitude)
                            map.center = coordinate
                            markerPopup.open()
                        }
                    }

                    Popup {
                        id: markerPopup
                        padding: 10
                        width: units.gu(25)
                        height: units.gu(35)
                        x: Math.round((parent.width - width) / 2)
                        y: Math.round((parent.height - height) / 2)
                        modal: true
                        focus: true
                        closePolicy: Popup.CloseOnEscape | Popup.CloseOnPressOutsideParent

                        contentItem: Text {
                            id: contentText
                            text: "Content goes here!"
                        }
                    }
                }
            }

            MouseArea {
                id: idModuleMouseDebug;
                acceptedButtons: Qt.LeftButton | Qt.RightButton
                parent: map
                anchors.fill: parent

                property variant lastCoordinate

                onPressed: {
                    map.lastX = mouse.x
                    map.lastY = mouse.y
                    map.pressX = mouse.x
                    map.pressY = mouse.y
                    lastCoordinate = map.toCoordinate(Qt.point(mouse.x, mouse.y))
                }

                onPositionChanged: {
                    if (mouse.button == Qt.LeftButton) {
                        map.lastX = mouse.x
                        map.lastY = mouse.y
                    }
                }

                onDoubleClicked: {
                    var mouseGeoPos = map.toCoordinate(Qt.point(mouse.x, mouse.y));
                    var preZoomPoint = map.fromCoordinate(mouseGeoPos, false);
                    if (mouse.button === Qt.LeftButton) {
                        map.zoomLevel = Math.floor(map.zoomLevel + 1)
                    } else if (mouse.button === Qt.RightButton) {
                        map.zoomLevel = Math.floor(map.zoomLevel - 1)
                    }
                    var postZoomPoint = map.fromCoordinate(mouseGeoPos, false);
                    var dx = postZoomPoint.x - preZoomPoint.x;
                    var dy = postZoomPoint.y - preZoomPoint.y;

                    var mapCenterPoint = Qt.point(map.width / 2.0 + dx, map.height / 2.0 + dy);
                    map.center = map.toCoordinate(mapCenterPoint);

                    map.lastX = -1;
                    map.lastY = -1;
                }

                onPressAndHold: {
                    var coordinate = map.toCoordinate(Qt.point(map.lastX,map.lastY))
                    var placeid = "Test: " + (locationModel.count + 1)
                    locationModel.append({lat: coordinate.latitude, lon: coordinate.longitude, color: "red", title: placeid, myid: locationModel.count})
                }
            }
        }

        PositionSource {
            id: positionSource
            active: true
            updateInterval: 1000
            preferredPositioningMethods: PositionSource.SatellitePositioningMethods

            onPositionChanged: {
                if(isNaN(position.coordinate.longitude) || isNaN(position.coordinate.latitude)) {
                    header.title = "Waiting for a GPS fix..."
                    return
                }

                map.center = position.coordinate
                header.title = "coords: " + position.coordinate.longitude + ", " + position.coordinate.latitude + ", " + map.zoomLevel
            }

            onSourceErrorChanged: {
                if (sourceError == PositionSource.NoError)
                    return

                console.log("Source error: " + sourceError)
            }
        }
    }

    Python {
        id: pytest
        Component.onCompleted: {
            addImportPath(Qt.resolvedUrl('../py/'))
            importModule("util", function() {});
        }
    }
}
